<?xml version="1.0" encoding="UTF-8"?>
<gui name="palapeli" version="5"
	xmlns="http://www.kde.org/standards/kxmlgui/1.0"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0 http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
	<MenuBar>
		<Menu name="game">
			<Action name="game_new"/>
			<Action name="game_import"/>
			<Separator/>
			<Action name="game_export"/>
			<Action name="game_delete"/>
			<Separator/>
			<Action name="game_restart"/>
		</Menu>
		<Menu name="move">
			<Action name="move_create_holder"/>
			<Action name="move_delete_holder"/>
			<Action name="move_select_all"/>
			<Action name="move_rearrange"/>
		</Menu>
		<Menu name="view">
			<Action name="view_collection"/>
			<Separator/>
			<Action name="view_preview"/>
			<Separator/>
			<Action name="view_zoom_in"/>
			<Action name="view_zoom_out"/>
		</Menu>
		<Menu name="settings">
			<Action name="enable_messages"/>
		</Menu>
	</MenuBar>
	<ToolBar name="mainToolBar">
		<text>Main toolbar</text>
		<Action name="game_new"/>
		<Action name="game_delete"/>
		<Separator/>
		<Action name="game_import"/>
		<Action name="game_export"/>
		<Separator/>
		<Action name="view_collection"/>
		<Action name="view_preview"/>
	</ToolBar>
</gui>
